package com.link.core.basic.es.parse;

import com.alibaba.fastjson.JSON;
import com.link.base.user.model.CoreUser;
import com.link.core.basic.es.service.BasicEsServiceImpl;
import com.link.core.basic.es.util.EsCacheUtils;
import com.link.core.basic.es.util.EsFieldType;
import com.link.core.basic.es.util.EsUtil;
import com.link.core.basic.model.BasicModel;
import com.link.core.basic.query.QueryParams;
import com.link.core.basic.security.OauthLevel;
import com.link.core.basic.service.BasicServiceException;
import com.link.core.basic.service.ServiceException;
import com.link.core.util.BeanTools;
import com.link.core.util.StringUtils;
import com.link.core.util.UserUtil;
import java.util.ArrayList;
import java.util.List;
import javax.annotation.Resource;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.elasticsearch.action.search.SearchRequest;
import org.elasticsearch.action.search.SearchResponse;
import org.elasticsearch.client.RequestOptions;
import org.elasticsearch.client.RestHighLevelClient;
import org.elasticsearch.index.query.BoolQueryBuilder;
import org.elasticsearch.index.query.QueryBuilder;
import org.elasticsearch.index.query.QueryBuilders;
import org.elasticsearch.search.SearchHits;
import org.elasticsearch.search.builder.SearchSourceBuilder;
import org.springframework.dao.EmptyResultDataAccessException;
import org.springframework.jdbc.BadSqlGrammarException;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Component;

@Component
public class EsOauthParse {
   private static final Logger logger = LogManager.getLogger(EsOauthParse.class);
   private static final String ALIAS_NAME_ORG_ID = "orgId";
   private static final String ALIAS_NAME_POSTN_ID = "postnId";
   private static final String ALIAS_NAME_MY = "createdBy";
   private static final String ES_PARTY_REL_INDEX = "es_lnk_party_rel";
   @Resource
   private JdbcTemplate jdbcTemplate;
   @Resource
   private EsCacheUtils esCacheUtils;
   @Resource
   private RestHighLevelClient restHighLevelClient;

   public QueryBuilder buildOauth(BasicModel basicModel) throws Exception {
      String oAuth = basicModel.getOauth();
      QueryParams qps = basicModel.getQps();
      if (qps != null) {
         oAuth = StringUtils.isNotBlank(qps.getOauth()) ? qps.getOauth() : basicModel.getOauth();
      }

      String oauth = this.oauthCheck(oAuth, basicModel.getTopOauth());
      if (StringUtils.isNull(oauth)) {
         return null;
      } else {
         QueryBuilder resultBuilder = null;
         CoreUser user = UserUtil.getUser();
         String menuId = null;
         String esKey;
         if (oauth.startsWith("ALL")) {
            menuId = oauth.substring(3);
         } else {
            String esFieldType;
            if (oauth.startsWith("MY_ORG_ONLY")) {
               esKey = "orgId";
               if (StringUtils.isNotBlank(basicModel.getOrgSecuritySourceId())) {
                  esKey = basicModel.getOrgSecuritySourceId();
               }

               esKey = (String)EsUtil.parseNestedField(esKey, basicModel).get(EsFieldType.NESTED_OBJECT.getValue());
               esFieldType = this.esCacheUtils.getFieldType((String)BasicEsServiceImpl.ES_INDEX.get(), new String[]{esKey});
               if (!EsFieldType.KEYWORD.getValue().equals(esFieldType)) {
                  esKey = esKey + ".keyword";
               }

               resultBuilder = QueryBuilders.termQuery(esKey, user.getOrgId());
               menuId = oauth.substring("MY_ORG_ONLY".length());
            } else if (oauth.startsWith("MY_ORG")) {
               esKey = "orgId";
               if (StringUtils.isNotBlank(basicModel.getOrgSecuritySourceId())) {
                  esKey = basicModel.getOrgSecuritySourceId();
               }

               esKey = (String)EsUtil.parseNestedField(esKey, basicModel).get(EsFieldType.NESTED_OBJECT.getValue());
               resultBuilder = this.createMyOrgSecurity(user.getOrgId(), esKey);
               menuId = oauth.substring("MY_ORG".length());
            } else if (oauth.startsWith("MY_POSTN_ONLY")) {
               esKey = "postnId";
               if (StringUtils.isNotBlank(basicModel.getPostnSecuritySourceId())) {
                  esKey = basicModel.getPostnSecuritySourceId();
               }

               esKey = (String)EsUtil.parseNestedField(esKey, basicModel).get(EsFieldType.NESTED_OBJECT.getValue());
               esFieldType = this.esCacheUtils.getFieldType((String)BasicEsServiceImpl.ES_INDEX.get(), new String[]{esKey});
               if (!EsFieldType.KEYWORD.getValue().equals(esFieldType)) {
                  esKey = esKey + ".keyword";
               }

               resultBuilder = QueryBuilders.termQuery(esKey, user.getPostnId());
               menuId = oauth.substring("MY_POSTN_ONLY".length());
            } else if (oauth.startsWith("MY_POSTN")) {
               esKey = "postnId";
               if (StringUtils.isNotBlank(basicModel.getPostnSecuritySourceId())) {
                  esKey = basicModel.getPostnSecuritySourceId();
               }

               esKey = (String)EsUtil.parseNestedField(esKey, basicModel).get(EsFieldType.NESTED_OBJECT.getValue());
               resultBuilder = this.createMyPostnSecurity(user.getPostnId(), esKey);
               menuId = oauth.substring("MY_POSTN".length());
            } else if (oauth.startsWith("MY")) {
               esKey = "createdBy";
               esFieldType = this.esCacheUtils.getFieldType((String)BasicEsServiceImpl.ES_INDEX.get(), new String[]{esKey});
               if (!EsFieldType.KEYWORD.getValue().equals(esFieldType)) {
                  esKey = esKey + ".keyword";
               }

               resultBuilder = QueryBuilders.termQuery(esKey, user.getId());
               menuId = oauth.substring("MY".length());
            }
         }

         esKey = this.querySecuritySql(menuId);
         basicModel.setEsOauth(esKey);
         return (QueryBuilder)resultBuilder;
      }
   }

   private QueryBuilder createMyOrgSecurity(String orgId, String esKey) throws Exception {
      SearchRequest searchOrgRequest = new SearchRequest();
      searchOrgRequest.indices(new String[]{"es_lnk_party_rel"});
      SearchSourceBuilder sourceOrgBuilder = new SearchSourceBuilder();
      BoolQueryBuilder queryBuilder = new BoolQueryBuilder();
      queryBuilder.must().add(QueryBuilders.termQuery("partyId", orgId));
      queryBuilder.must().add(QueryBuilders.termQuery("dataType", "ORG"));
      sourceOrgBuilder.query(queryBuilder);
      sourceOrgBuilder.size(10000);
      searchOrgRequest.source(sourceOrgBuilder);
      SearchResponse searchResponse = this.restHighLevelClient.search(searchOrgRequest, RequestOptions.DEFAULT);
      SearchHits hits = searchResponse.getHits();
      List<String> subOrgIds = new ArrayList();
      hits.forEach((p) -> {
         try {
            String subOrgId = JSON.parseObject(p.getSourceAsString()).getString("subPartyId");
            if (StringUtils.isNotBlank(subOrgId)) {
               subOrgIds.add(subOrgId);
            }

         } catch (Exception var3) {
            Exception e = var3;
            throw new BasicServiceException("查询ES下级组织数据异常", e);
         }
      });
      String esFieldType = this.esCacheUtils.getFieldType((String)BasicEsServiceImpl.ES_INDEX.get(), new String[]{esKey});
      if (!EsFieldType.KEYWORD.getValue().equals(esFieldType)) {
         esKey = esKey + ".keyword";
      }

      return QueryBuilders.termsQuery(esKey, subOrgIds);
   }

   private QueryBuilder createMyPostnSecurity(String postnId, String esKey) throws Exception {
      SearchRequest searchPostnRequest = new SearchRequest();
      searchPostnRequest.indices(new String[]{"es_lnk_party_rel"});
      SearchSourceBuilder sourcePostnBuilder = new SearchSourceBuilder();
      BoolQueryBuilder queryBuilder = new BoolQueryBuilder();
      queryBuilder.must().add(QueryBuilders.termQuery("partyId", postnId));
      queryBuilder.must().add(QueryBuilders.termQuery("dataType", "POSTN"));
      searchPostnRequest.source(sourcePostnBuilder);
      sourcePostnBuilder.query(queryBuilder);
      sourcePostnBuilder.size(10000);
      SearchResponse searchResponse = this.restHighLevelClient.search(searchPostnRequest, RequestOptions.DEFAULT);
      SearchHits hits = searchResponse.getHits();
      List<String> subPostnIds = new ArrayList();
      hits.forEach((p) -> {
         try {
            String subPostnId = JSON.parseObject(p.getSourceAsString()).getString("subPartyId");
            if (StringUtils.isNotBlank(subPostnId)) {
               subPostnIds.add(subPostnId);
            }

         } catch (Exception var3) {
            Exception e = var3;
            throw new BasicServiceException("查询ES下级职位数据异常", e);
         }
      });
      String esFieldType = this.esCacheUtils.getFieldType((String)BasicEsServiceImpl.ES_INDEX.get(), new String[]{esKey});
      if (!EsFieldType.KEYWORD.getValue().equals(esFieldType)) {
         esKey = esKey + ".keyword";
      }

      return QueryBuilders.termsQuery(esKey, subPostnIds);
   }

   public String oauthCheck(String oauth, String topOauth) {
      if (StringUtils.isBlank(oauth)) {
         return topOauth;
      } else {
         String oauthPrefix = null;
         String menuId = null;
         if (oauth.startsWith("ALL")) {
            oauthPrefix = oauth.substring(0, 3);
            menuId = oauth.substring(3);
         } else if (oauth.startsWith("MY_ORG_ONLY")) {
            oauthPrefix = oauth.substring(0, 11);
            menuId = oauth.substring(11);
         } else if (oauth.startsWith("MY_ORG")) {
            oauthPrefix = oauth.substring(0, 6);
            menuId = oauth.substring(6);
         } else if (oauth.startsWith("MY_POSTN_ONLY")) {
            oauthPrefix = oauth.substring(0, 13);
            menuId = oauth.substring(13);
         } else if (oauth.startsWith("MY_POSTN")) {
            oauthPrefix = oauth.substring(0, 8);
            menuId = oauth.substring(8);
         } else if (oauth.startsWith("MY")) {
            oauthPrefix = oauth.substring(0, 2);
            menuId = oauth.substring(2);
         } else if (oauth.startsWith("ACCESS_GROUP")) {
            oauthPrefix = oauth.substring(0, 12);
            menuId = oauth.substring(12);
         }

         if (!StringUtils.isBlank(menuId)) {
            return oauth;
         } else if (StringUtils.isBlank(oauthPrefix)) {
            return topOauth;
         } else {
            Integer oauthPrefixLevel = (Integer)OauthLevel.getOauthLevel().get(oauthPrefix);
            Integer topOauthLevel = (Integer)OauthLevel.getOauthLevel().get(topOauth);
            if (null != oauthPrefixLevel && null != topOauthLevel) {
               if (oauthPrefixLevel < topOauthLevel) {
                  oauth = topOauth;
               }

               return oauth;
            } else {
               return topOauth;
            }
         }
      }
   }

   public String querySecuritySql(String menuId) {
      if (StringUtils.isNull(menuId)) {
         return null;
      } else if (StringUtils.haveIllegalSearchCharacters(menuId)) {
         logger.error("获取安全性菜单参数menuId：[" + menuId + "]包含非法字符，[-- ] [/*] ['] [#]");
         throw new ServiceException("INTERCEPTOR-013");
      } else {
         String sql = "SELECT OAUTH_SQL AS result FROM LNK_MENU WHERE ROW_ID = '" + menuId + "'";
         if (this.jdbcTemplate == null) {
            this.jdbcTemplate = (JdbcTemplate)BeanTools.getBean(JdbcTemplate.class);
         }

         try {
            assert this.jdbcTemplate != null;

            String oauthSql = (String)this.jdbcTemplate.queryForObject(sql, String.class);
            return oauthSql;
         } catch (EmptyResultDataAccessException var5) {
            EmptyResultDataAccessException e = var5;
            logger.error("获取安全性信息失败[" + menuId + "]");
            logger.error(e);
            throw new ServiceException("INTERCEPTOR-014");
         } catch (BadSqlGrammarException var6) {
            logger.error("获取安全性菜单参数menuId被窜改，篡改后的参数为[" + menuId + "]");
            throw new ServiceException("INTERCEPTOR-013");
         }
      }
   }
}
